@@ -9,11 +9,12 @@ bootstrap issues (/usr/bin/python is Python 2.3 on OSX 10.4)
 
 Usage: see USAGE variable in the script.
 """
-import platform, os, sys, getopt, textwrap, shutil, urllib2, stat, time, pwd
-import grp, md5
+import platform, os, sys, getopt, textwrap, shutil, stat, time, pwd
+import urllib.request
+import grp
 
-INCLUDE_TIMESTAMP=1
-VERBOSE=1
+INCLUDE_TIMESTAMP = 1
+VERBOSE = 1
 
 from plistlib import Plist
 
@@ -31,8 +32,6 @@ except ImportError:
     def writePlist(plist, path):
         plist.write(path)
 
-
-
 def shellQuote(value):
     """
     Return the string value in a form that can safely be inserted into
@@ -56,30 +55,30 @@ def getFullVersion():
         if 'PY_VERSION' in ln:
             return ln.split()[-1][1:-1]
 
-    raise RuntimeError, "Cannot find full version??"
+    raise RuntimeError("Cannot find full version??")
 
 # The directory we'll use to create the build (will be erased and recreated)
-WORKDIR="/tmp/_py"
+WORKDIR = "/tmp/_py"
 
 # The directory we'll use to store third-party sources. Set this to something
 # else if you don't want to re-fetch required libraries every time.
-DEPSRC=os.path.join(WORKDIR, 'third-party')
-DEPSRC=os.path.expanduser('/tmp/other-sources')
+DEPSRC = os.path.join(WORKDIR, 'third-party')
+DEPSRC = os.path.expanduser('~/Universal/other-sources')
 
 # Location of the preferred SDK
-SDKPATH="/Developer/SDKs/MacOSX10.4u.sdk"
-#SDKPATH="/"
+SDKPATH = "/Developer/SDKs/MacOSX10.4u.sdk"
+#SDKPATH = "/"
 
-ARCHLIST=('i386', 'ppc',)
+ARCHLIST = ('i386', 'ppc',)
 
 # Source directory (asume we're in Mac/BuildScript)
-SRCDIR=os.path.dirname(
+SRCDIR = os.path.dirname(
         os.path.dirname(
             os.path.dirname(
                 os.path.abspath(__file__
         ))))
 
-USAGE=textwrap.dedent("""\
+USAGE = textwrap.dedent("""\
     Usage: build_python [options]
 
     Options:
@@ -94,11 +93,10 @@ USAGE=textwrap.dedent("""\
 
 # Instructions for building libraries that are necessary for building a
 # batteries included python.
-LIBRARY_RECIPES=[
+LIBRARY_RECIPES = [
     dict(
-        name="Bzip2 1.0.4",
-        url="http://www.bzip.org/1.0.4/bzip2-1.0.4.tar.gz",
-        checksum='fc310b254f6ba5fbb5da018f04533688',
+        name="Bzip2 1.0.3",
+        url="http://www.bzip.org/1.0.3/bzip2-1.0.3.tar.gz",
         configure=None,
         install='make install PREFIX=%s/usr/local/ CFLAGS="-arch %s -isysroot %s"'%(
             shellQuote(os.path.join(WORKDIR, 'libraries')),
@@ -109,7 +107,6 @@ LIBRARY_RECIPES=[
     dict(
         name="ZLib 1.2.3",
         url="http://www.gzip.org/zlib/zlib-1.2.3.tar.gz",
-        checksum='debc62758716a169df9f62e6ab2bc634',
         configure=None,
         install='make install prefix=%s/usr/local/ CFLAGS="-arch %s -isysroot %s"'%(
             shellQuote(os.path.join(WORKDIR, 'libraries')),
@@ -122,7 +119,6 @@ LIBRARY_RECIPES=[
         name="GNU Readline 5.1.4",
         url="http://ftp.gnu.org/pub/gnu/readline/readline-5.1.tar.gz" ,
         patchlevel='0',
-        checksum='7ee5a692db88b30ca48927a13fd60e46',
         patches=[
             # The readline maintainers don't do actual micro releases, but
             # just ship a set of patches.
@@ -134,9 +130,9 @@ LIBRARY_RECIPES=[
     ),
 
     dict(
-        name="SQLite 3.3.14",
-        url="http://www.sqlite.org/sqlite-3.3.14.tar.gz",
-        checksum='e1a4428a5cb17f28164731b72f06130a',
+        name="SQLite 3.6.3",
+        url="http://www.sqlite.org/sqlite-3.6.3.tar.gz",
+        checksum='93f742986e8bc2dfa34792e16df017a6feccf3a2',
         configure_pre=[
             '--enable-threadsafe',
             '--enable-tempstore',
@@ -149,7 +145,6 @@ LIBRARY_RECIPES=[
     dict(
         name="NCurses 5.5",
         url="http://ftp.gnu.org/pub/gnu/ncurses/ncurses-5.5.tar.gz",
-        checksum='e73c1ac10b4bfc46db43b2ddfd6244ef',
         configure_pre=[
             "--without-cxx",
             "--without-ada",
@@ -176,9 +171,8 @@ LIBRARY_RECIPES=[
             ),
     ),
     dict(
-        name="Sleepycat DB 4.4",
-        url="http://downloads.sleepycat.com/db-4.4.20.tar.gz",
-        checksum='d84dff288a19186b136b0daf7067ade3',
+        name="Sleepycat DB 4.7.25",
+        url="http://download.oracle.com/berkeley-db/db-4.7.25.tar.gz",
         #name="Sleepycat DB 4.3.29",
         #url="http://downloads.sleepycat.com/db-4.3.29.tar.gz",
         buildDir="build_unix",
@@ -191,7 +185,7 @@ LIBRARY_RECIPES=[
 
 
 # Instructions for building packages inside the .mpkg.
-PKG_RECIPES=[
+PKG_RECIPES = [
     dict(
         name="PythonFramework",
         long_name="Python Framework",
@@ -206,7 +200,7 @@ PKG_RECIPES=[
     dict(
         name="PythonApplications",
         long_name="GUI Applications",
-        source="/Applications/MacPython %(VER)s",
+        source="/Applications/Python %(VER)s",
         readme="""\
             This package installs IDLE (an interactive Python IDE),
             Python Launcher and Build Applet (create application bundles
@@ -296,9 +290,9 @@ def runCommand(commandline):
     fd = os.popen(commandline, 'r')
     data = fd.read()
     xit = fd.close()
-    if xit != None:
+    if xit is not None:
         sys.stdout.write(data)
-        raise RuntimeError, "command failed: %s"%(commandline,)
+        raise RuntimeError("command failed: %s"%(commandline,))
 
     if VERBOSE:
         sys.stdout.write(data); sys.stdout.flush()
@@ -307,9 +301,9 @@ def captureCommand(commandline):
     fd = os.popen(commandline, 'r')
     data = fd.read()
     xit = fd.close()
-    if xit != None:
+    if xit is not None:
         sys.stdout.write(data)
-        raise RuntimeError, "command failed: %s"%(commandline,)
+        raise RuntimeError("command failed: %s"%(commandline,))
 
     return data
 
@@ -328,20 +322,9 @@ def checkEnvironment():
         fatal("Please install the latest version of Xcode and the %s SDK"%(
             os.path.basename(SDKPATH[:-4])))
 
-    if os.path.exists('/sw'):
-        fatal("Detected Fink, please remove before building Python")
-
-    if os.path.exists('/opt/local'):
-        fatal("Detected MacPorts, please remove before building Python")
-
-    if not os.path.exists('/Library/Frameworks/Tcl.framework') or \
-            not os.path.exists('/Library/Frameworks/Tk.framework'):
-
-        fatal("Please install a Universal Tcl/Tk framework in /Library from\n\thttp://tcltkaqua.sourceforge.net/")
 
 
-
-def parseOptions(args = None):
+def parseOptions(args=None):
     """
     Parse arguments and update global settings.
     """
@@ -353,17 +336,17 @@ def parseOptions(args = None):
     try:
         options, args = getopt.getopt(args, '?hb',
                 [ 'build-dir=', 'third-party=', 'sdk-path=' , 'src-dir='])
-    except getopt.error, msg:
-        print msg
+    except getopt.error as msg:
+        print(msg)
         sys.exit(1)
 
     if args:
-        print "Additional arguments"
+        print("Additional arguments")
         sys.exit(1)
 
     for k, v in options:
         if k in ('-h', '-?'):
-            print USAGE
+            print(USAGE)
             sys.exit(0)
 
         elif k in ('-d', '--build-dir'):
@@ -379,19 +362,19 @@ def parseOptions(args = None):
             SRCDIR=v
 
         else:
-            raise NotImplementedError, k
+            raise NotImplementedError(k)
 
     SRCDIR=os.path.abspath(SRCDIR)
     WORKDIR=os.path.abspath(WORKDIR)
     SDKPATH=os.path.abspath(SDKPATH)
     DEPSRC=os.path.abspath(DEPSRC)
 
-    print "Settings:"
-    print " * Source directory:", SRCDIR
-    print " * Build directory: ", WORKDIR
-    print " * SDK location:    ", SDKPATH
-    print " * third-party source:", DEPSRC
-    print ""
+    print("Settings:")
+    print(" * Source directory:", SRCDIR)
+    print(" * Build directory: ", WORKDIR)
+    print(" * SDK location:    ", SDKPATH)
+    print(" * third-party source:", DEPSRC)
+    print("")
 
 
 
@@ -436,7 +419,7 @@ def extractArchive(builddir, archiveName):
         xit = fp.close()
         if xit is not None:
             sys.stdout.write(data)
-            raise RuntimeError, "Cannot extract %s"%(archiveName,)
+            raise RuntimeError("Cannot extract %s"%(archiveName,))
 
         return os.path.join(builddir, retval)
 
@@ -458,9 +441,9 @@ def downloadURL(url, fname):
         pass
     else:
         if KNOWNSIZES.get(url) == size:
-            print "Using existing file for", url
+            print("Using existing file for", url)
             return
-    fpIn = urllib2.urlopen(url)
+    fpIn = urllib.request.urlopen(url)
     fpOut = open(fname, 'wb')
     block = fpIn.read(10240)
     try:
@@ -475,17 +458,6 @@ def downloadURL(url, fname):
         except:
             pass
 
-
-def verifyChecksum(path, checksum):
-    summer = md5.md5()
-    fp = open(path, 'rb')
-    block = fp.read(10240)
-    while block:
-        summer.update(block)
-        block = fp.read(10240)
-
-    return summer.hexdigest() == checksum
-
 def buildRecipe(recipe, basedir, archList):
     """
     Build software using a recipe. This function does the
@@ -507,17 +479,15 @@ def buildRecipe(recipe, basedir, archList):
         os.mkdir(DEPSRC)
 
 
-    if os.path.exists(sourceArchive) and verifyChecksum(sourceArchive, recipe['checksum']):
-        print "Using local copy of %s"%(name,)
+    if os.path.exists(sourceArchive):
+        print("Using local copy of %s"%(name,))
 
     else:
-        print "Downloading %s"%(name,)
+        print("Downloading %s"%(name,))
         downloadURL(url, sourceArchive)
-        print "Archive for %s stored as %s"%(name, sourceArchive)
-        if not verifyChecksum(sourceArchive, recipe['checksum']):
-            fatal("Download for %s failed: bad checksum"%(url,))
+        print("Archive for %s stored as %s"%(name, sourceArchive))
 
-    print "Extracting archive for %s"%(name,)
+    print("Extracting archive for %s"%(name,))
     buildDir=os.path.join(WORKDIR, '_bld')
     if not os.path.exists(buildDir):
         os.mkdir(buildDir)
@@ -580,14 +550,14 @@ def buildRecipe(recipe, basedir, archList):
         configure_args.insert(0, configure)
         configure_args = [ shellQuote(a) for a in configure_args ]
 
-        print "Running configure for %s"%(name,)
+        print("Running configure for %s"%(name,))
         runCommand(' '.join(configure_args) + ' 2>&1')
 
-    print "Running install for %s"%(name,)
+    print("Running install for %s"%(name,))
     runCommand('{ ' + install + ' ;} 2>&1')
 
-    print "Done %s"%(name,)
-    print ""
+    print("Done %s"%(name,))
+    print("")
 
     os.chdir(curdir)
 
@@ -595,9 +565,9 @@ def buildLibraries():
     """
     Build our dependencies into $WORKDIR/libraries/usr/local
     """
-    print ""
-    print "Building required libraries"
-    print ""
+    print("")
+    print("Building required libraries")
+    print("")
     universal = os.path.join(WORKDIR, 'libraries')
     os.mkdir(universal)
     os.makedirs(os.path.join(universal, 'usr', 'local', 'lib'))
@@ -611,31 +581,33 @@ def buildLibraries():
 def buildPythonDocs():
     # This stores the documentation as Resources/English.lproj/Docuentation
     # inside the framwork. pydoc and IDLE will pick it up there.
-    print "Install python documentation"
+    print("Install python documentation")
     rootDir = os.path.join(WORKDIR, '_root')
     version = getVersion()
     docdir = os.path.join(rootDir, 'pydocs')
 
+    novername = 'python-docs-html.tar.bz2'
     name = 'html-%s.tar.bz2'%(getFullVersion(),)
     sourceArchive = os.path.join(DEPSRC, name)
     if os.path.exists(sourceArchive):
-        print "Using local copy of %s"%(name,)
+        print("Using local copy of %s"%(name,))
 
     else:
-        print "Downloading %s"%(name,)
+        print "Downloading %s"%(novername,)
         downloadURL('http://www.python.org/ftp/python/doc/%s/%s'%(
-            getFullVersion(), name), sourceArchive)
-        print "Archive for %s stored as %s"%(name, sourceArchive)
+            getFullVersion(), novername), sourceArchive)
+        print("Archive for %s stored as %s"%(name, sourceArchive))
 
     extractArchive(os.path.dirname(docdir), sourceArchive)
+
     os.rename(
             os.path.join(
-                os.path.dirname(docdir), 'Python-Docs-%s'%(getFullVersion(),)),
+                os.path.dirname(docdir), 'python-docs-html'),
             docdir)
 
 
 def buildPython():
-    print "Building a universal python"
+    print("Building a universal python")
 
     buildDir = os.path.join(WORKDIR, '_bld', 'python')
     rootDir = os.path.join(WORKDIR, '_root')
@@ -658,24 +630,24 @@ def buildPython():
     # several paths.
     version = getVersion()
 
-    print "Running configure..."
+    print("Running configure...")
     runCommand("%s -C --enable-framework --enable-universalsdk=%s LDFLAGS='-g -L%s/libraries/usr/local/lib' OPT='-g -O3 -I%s/libraries/usr/local/include' 2>&1"%(
         shellQuote(os.path.join(SRCDIR, 'configure')),
         shellQuote(SDKPATH), shellQuote(WORKDIR)[1:-1],
         shellQuote(WORKDIR)[1:-1]))
 
-    print "Running make"
+    print("Running make")
     runCommand("make")
 
-    print "Running make frameworkinstall"
+    print("Running make frameworkinstall")
     runCommand("make frameworkinstall DESTDIR=%s"%(
         shellQuote(rootDir)))
 
-    print "Running make frameworkinstallextras"
+    print("Running make frameworkinstallextras")
     runCommand("make frameworkinstallextras DESTDIR=%s"%(
         shellQuote(rootDir)))
 
-    print "Copying required shared libraries"
+    print("Copying required shared libraries")
     if os.path.exists(os.path.join(WORKDIR, 'libraries', 'Library')):
         runCommand("mv %s/* %s"%(
             shellQuote(os.path.join(
@@ -686,7 +658,7 @@ def buildPython():
                 'Python.framework', 'Versions', getVersion(),
                 'lib'))))
 
-    print "Fix file modes"
+    print("Fix file modes")
     frmDir = os.path.join(rootDir, 'Library', 'Frameworks', 'Python.framework')
     gid = grp.getgrnam('admin').gr_gid
 
@@ -695,6 +667,7 @@ def buildPython():
             os.chmod(os.path.join(dirpath, dn), 0775)
             os.chown(os.path.join(dirpath, dn), -1, gid)
 
+
         for fn in filenames:
             if os.path.islink(fn):
                 continue
@@ -733,16 +706,6 @@ def buildPython():
         os.symlink(os.path.join(to_framework, fn),
                    os.path.join(usr_local_bin, fn))
 
-    # A quick fix that removes a spurious unittest failure when users
-    # upgrade their python2.5 install and then run Python's test suite.
-    # This is needed because one of the test files for the decimal module
-    # changed it name (see issue 2114).
-    test_dir = os.path.join(rootDir, 'Library', 'Frameworks',
-            'Python.framework', 'Versions', version, 'lib',
-            'python%s'%(version,), 'test', 'decimaltestdata')
-    data = open(os.path.join(test_dir, 'reduce.decTest'), 'r').read()
-    open(os.path.join(test_dir, 'normalize.decTest'), 'w').write(data)
-
     os.chdir(curdir)
 
 
@@ -784,7 +747,7 @@ def packageFromRecipe(targetDir, recipe):
         readme = textwrap.dedent(recipe['readme'])
         isRequired = recipe.get('required', True)
 
-        print "- building package %s"%(pkgname,)
+        print("- building package %s"%(pkgname,))
 
         # Substitute some variables
         textvars = dict(
@@ -1050,7 +1013,6 @@ def setIcon(filePath, icnsPath):
             shellQuote(tmpPath),
         ))
 
-
 def main():
     # First parse options and check if we can perform our work
     parseOptions()
@@ -1069,11 +1031,11 @@ def main():
     buildPython()
     buildPythonDocs()
     fn = os.path.join(WORKDIR, "_root", "Applications",
-                "MacPython %s"%(getVersion(),), "Update Shell Profile.command")
+                "Python %s"%(getVersion(),), "Update Shell Profile.command")
     patchFile("scripts/postflight.patch-profile",  fn)
     os.chmod(fn, 0755)
 
-    folder = os.path.join(WORKDIR, "_root", "Applications", "MacPython %s"%(
+    folder = os.path.join(WORKDIR, "_root", "Applications", "Python %s"%(
         getVersion(),))
     os.chmod(folder, 0755)
     setIcon(folder, "../Icons/Python Folder.icns")
@@ -1088,9 +1050,9 @@ def main():
     shutil.copy('../../LICENSE', os.path.join(WORKDIR, 'installer', 'License.txt'))
 
     fp = open(os.path.join(WORKDIR, 'installer', 'Build.txt'), 'w')
-    print >> fp, "# BUILD INFO"
-    print >> fp, "# Date:", time.ctime()
-    print >> fp, "# By:", pwd.getpwuid(os.getuid()).pw_gecos
+    print("# BUILD INFO", file=fp)
+    print("# Date:", time.ctime(), file=fp)
+    print("# By:", pwd.getpwuid(os.getuid()).pw_gecos, file=fp)
     fp.close()
 
     # Custom icon for the DMG, shown when the DMG is mounted.
@@ -1103,5 +1065,6 @@ def main():
     # And copy it to a DMG
     buildDMG()
 
+
 if __name__ == "__main__":
     main()
